/*******************************************************************************
 *                           LIBERERIA LCD MODIFICADA
 * 
 * - Asignacin de pines independientes al puerto
 * - Funcionamiento en bus de 8 y 4 bits
 * - Definicion del numero de filas y columnas del display
 * - Algunas funciones como SCROLL
*******************************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include "config.h"     //Necesaria porque es donde esta definido el _XTAL_FREQ


////////////////////////////////////////////////////////////////////////////////
//                  FUNCION ENCARGADA DE INICIALIZAR EL SISTEMA
//
//  Configura los pines como salida
//  Configura el display en el modo correcto
////////////////////////////////////////////////////////////////////////////////
void LCD_Initialize(void);


////////////////////////////////////////////////////////////////////////////////
//                  FUNCION ENCARGADA DE HACER SCROLL DEL TEXTO
//
////////////////////////////////////////////////////////////////////////////////
void LCD_ScrollLeft (void);


void LCD_ScrollRight (void);



////////////////////////////////////////////////////////////////////////////////
//                  FUNCION ENCARGADA PONER CURSOR EN UNA POSICION
//
////////////////////////////////////////////////////////////////////////////////
void LCD_SetCursor ( uint8_t linea, uint8_t columna );



////////////////////////////////////////////////////////////////////////////////
//                  FUNCION ENCARGADA PONER CURSOR EN UNA POSICION
//
////////////////////////////////////////////////////////////////////////////////
void LCD_SetCursor (uint8_t linea, uint8_t columna );



////////////////////////////////////////////////////////////////////////////////
//                  FUNCION ENCARGADA DE ESCRIBIR UNA CADENA
//
////////////////////////////////////////////////////////////////////////////////
void LCD_PutString ( char* inputString , uint16_t length );



////////////////////////////////////////////////////////////////////////////////
//                  FUNCION ENCARGADA DE ESCRIBIR CARACTERES
//
// Recibe caracteres y cambia de linea automaticamente
////////////////////////////////////////////////////////////////////////////////
void LCD_PutChar(char);



////////////////////////////////////////////////////////////////////////////////
//                  FUNCION ENCARGADA DE BORRAR DISPLAY
//
//  Borra display
//  Salta a la posicion inicial
////////////////////////////////////////////////////////////////////////////////
void LCD_ClearScreen(void);



////////////////////////////////////////////////////////////////////////////////
//                  FUNCION ENCARGADA DE HACER VISIBLE O NO EL CURSOR
//
////////////////////////////////////////////////////////////////////////////////
void LCD_CursorEnable(bool enable);



////////////////////////////////////////////////////////////////////////////////
//                  FUNCION ENCARGADA DE ENVIAR DATOS AL DISPLAY
//
//  Segun el tipo de configuracion 4bits o 8bits
////////////////////////////////////////////////////////////////////////////////
static void LCD_SendData ( char data );



////////////////////////////////////////////////////////////////////////////////
//                  FUNCION ENCARGADA DE ENVIAR COMANDOS AL DISPLAY
//
//  Segun el tipo de configuracion 4bits o 8bits
////////////////////////////////////////////////////////////////////////////////
static void LCD_SendCommand ( char command , unsigned int delay );



////////////////////////////////////////////////////////////////////////////////
//                  FUNCION ENCARGADA DE HACER RETARDO uS
//
////////////////////////////////////////////////////////////////////////////////
static void LCD_Wait_us ( unsigned int delay );